--
-- CustomTrackSystem
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2015-03-01 - Initial implementation
--				v1.1 - 2015-09-15 - Update
-- 

CustomTrackSystem = {};

function CustomTrackSystem.prerequisitesPresent(specializations)
    return true;
end;

function CustomTrackSystem:load(xmlFile)
	local file = getXMLString(xmlFile, "vehicle.CustomTrackSystem#file");
	if file ~= nil and file ~= "" and g_currentMission.tyreTrackSystem ~= nil then
		file = Utils.getFilename(file, self.baseDirectory);
		
		if fileExists(file) then
			local system = TyreTrackSystem:new();
			system:load(file);
			self.customTrackSystem = system;
		end;
	end;
	
	if self.customTrackSystem ~= nil then
		self.hasWheelContactUpdate = {};
		self.trackSystemData = {};
		
		local i = 0;
		while true do
			local key = string.format("vehicle.wheels.wheel(%d)", i);
			if not hasXMLProperty(xmlFile, key) then break; end;
			
			local customTrackAtlasIndex = getXMLInt(xmlFile, key .. "#customTrackAtlasIndex");
			if customTrackAtlasIndex ~= nil then
				self.wheels[i + 1].customTrackAtlasIndex = customTrackAtlasIndex;
				
				local entry = {};
				entry.id = i + 1;
				entry.trackIndex = self.customTrackSystem:createTrack(self.wheels[i + 1].width, customTrackAtlasIndex);
				
				table.insert(self.trackSystemData, entry);
				self.hasWheelContactUpdate[i + 1] = self.wheels[i + 1].tyreTrackIndex == nil;
				
				if not self.hasWheelContactUpdate[i + 1] then
					print("CustomTrackSystem - Warning: You can't use the default track system along with CustomTrackSystem on wheel " .. i + 1 .. ", CustomTrackSystem have been deactivated for this wheel.");
				end;
			end;
			
			i = i + 1;
		end;
	end;
end;

function CustomTrackSystem:delete()
	if self.customTrackSystem ~= nil then
		for _, v in ipairs(self.trackSystemData) do
			self.customTrackSystem:destroyTrack(v.trackIndex);
		end;
		
		self.customTrackSystem:delete();
	end;
end;

function CustomTrackSystem:readStream(streamId, connection)
	if self.trackSystemData ~= nil and connection:getIsServer() then
		for _, v in ipairs(self.trackSystemData) do
			if self.hasWheelContactUpdate[v.id] then
				self.wheels[v.id].contact = streamReadInt8(streamId);
			end;
		end;
	end;
end;

function CustomTrackSystem:writeStream(streamId, connection)
	if self.trackSystemData ~= nil and not connection:getIsServer() then
		for _, v in ipairs(self.trackSystemData) do
			if self.hasWheelContactUpdate[v.id] then
				streamWriteInt8(streamId, self.wheels[v.id].contact);
			end;
		end;
	end;
end;

function CustomTrackSystem:readUpdateStream(streamId, timestamp, connection)
	CustomTrackSystem.readStream(self, streamId, connection);
end;

function CustomTrackSystem:writeUpdateStream(streamId, connection, dirtyMask)
	CustomTrackSystem.writeStream(self, streamId, connection);
end;

function CustomTrackSystem:mouseEvent(posX, posY, isDown, isUp, button)
end;

function CustomTrackSystem:keyEvent(unicode, sym, modifier, isDown)
end;

function CustomTrackSystem:update(dt)
end;

function CustomTrackSystem:updateTick(dt)
	if self.customTrackSystem ~= nil then
		if self:getIsActive() then
			for _, v in ipairs(self.trackSystemData) do
				if self.hasWheelContactUpdate[v.id] then
					local wheel = self.wheels[v.id];
					local wheelSpeed = 0;
					local color;
					
					if self.isServer then
						wheelSpeed = getWheelShapeAxleSpeed(wheel.node, wheel.wheelShape);
					else
						wheelSpeed = 1;
					end;
					
					if wheel.contact == Vehicle.WHEEL_GROUND_CONTACT then
						color = wheel.lastColor;
					elseif wheel.contact == Vehicle.WHEEL_OBJ_CONTACT then
						if wheel.dirtAmount > 0 then
							color = wheel.lastColor;
							color[4] = 0; -- no depth to tyre tracks on road etc.
						end;
					end;
					
					if color ~= nil then
						local ux, uy, uz = localDirectionToWorld(self.rootNode, 0, 1, 0);
						local wx, wy, wz = worldToLocal(wheel.node, getWorldTranslation(wheel.driveNode));
						wy = wy - wheel.radius;
						wx = wx + wheel.xoffset;
						wx, wy, wz = localToWorld(wheel.node, wx, wy, wz);
					
						self.customTrackSystem:addTrackPoint(v.trackIndex, wx, wy, wz, ux, uy, uz, wheel.lastColor[1], wheel.lastColor[2], wheel.lastColor[3], wheel.dirtAmount, color[4], wheelSpeed);
					else
						self.customTrackSystem:cutTrack(v.trackIndex);
					end;
				end;
			end;
		end;
	end;
end;

function CustomTrackSystem:draw()
end;